@@ -4,15 +4,15 @@
 use async_trait::async_trait;
 use error::Result;
 use std::collections::HashMap;
+use std::future::Future;
+use std::pin::Pin;
 use std::sync::Arc;
 
 use stream_info::StreamInfo;
 
-#[cfg(test)]
-pub(crate) mod mock;
-
 pub mod chain;
 mod error;
+pub mod mock;
 pub mod nack;
 pub mod noop;
 pub mod registry;
@@ -23,6 +23,9 @@ pub mod twcc;
 
 pub use error::Error;
 
+/// Attributes are a generic key/value store used by interceptors
+pub type Attributes = HashMap<usize, usize>;
+
 /// InterceptorBuilder provides an interface for constructing interceptors
 pub trait InterceptorBuilder {
     fn build(&self, id: &str) -> Result<Arc<dyn Interceptor + Send + Sync>>;
@@ -78,6 +81,24 @@ pub trait RTPWriter {
     async fn write(&self, pkt: &rtp::packet::Packet, attributes: &Attributes) -> Result<usize>;
 }
 
+pub type RTPWriterBoxFn = Box<
+    dyn (Fn(
+            &rtp::packet::Packet,
+            &Attributes,
+        ) -> Pin<Box<dyn Future<Output = Result<usize>> + Send + Sync>>)
+        + Send
+        + Sync,
+>;
+pub struct RTPWriterFn(pub RTPWriterBoxFn);
+
+#[async_trait]
+impl RTPWriter for RTPWriterFn {
+    /// write a rtp packet
+    async fn write(&self, pkt: &rtp::packet::Packet, attributes: &Attributes) -> Result<usize> {
+        self.0(pkt, attributes).await
+    }
+}
+
 /// RTPReader is used by Interceptor.bind_remote_stream.
 #[async_trait]
 pub trait RTPReader {
@@ -85,17 +106,58 @@ pub trait RTPReader {
     async fn read(&self, buf: &mut [u8], attributes: &Attributes) -> Result<(usize, Attributes)>;
 }
 
+pub type RTPReaderBoxFn = Box<
+    dyn (Fn(
+            &mut [u8],
+            &Attributes,
+        ) -> Pin<Box<dyn Future<Output = Result<(usize, Attributes)>> + Send + Sync>>)
+        + Send
+        + Sync,
+>;
+pub struct RTPReaderFn(pub RTPReaderBoxFn);
+
+#[async_trait]
+impl RTPReader for RTPReaderFn {
+    /// read a rtp packet
+    async fn read(&self, buf: &mut [u8], attributes: &Attributes) -> Result<(usize, Attributes)> {
+        self.0(buf, attributes).await
+    }
+}
+
 /// RTCPWriter is used by Interceptor.bind_rtcpwriter.
 #[async_trait]
 pub trait RTCPWriter {
     /// write a batch of rtcp packets
     async fn write(
         &self,
-        pkt: &(dyn rtcp::packet::Packet + Send + Sync),
+        pkts: &[Box<dyn rtcp::packet::Packet + Send + Sync>],
         attributes: &Attributes,
     ) -> Result<usize>;
 }
 
+pub type RTCPWriterBoxFn = Box<
+    dyn (Fn(
+            &[Box<dyn rtcp::packet::Packet + Send + Sync>],
+            &Attributes,
+        ) -> Pin<Box<dyn Future<Output = Result<usize>> + Send + Sync>>)
+        + Send
+        + Sync,
+>;
+
+pub struct RTCPWriterFn(pub RTCPWriterBoxFn);
+
+#[async_trait]
+impl RTCPWriter for RTCPWriterFn {
+    /// write a batch of rtcp packets
+    async fn write(
+        &self,
+        pkts: &[Box<dyn rtcp::packet::Packet + Send + Sync>],
+        attributes: &Attributes,
+    ) -> Result<usize> {
+        self.0(pkts, attributes).await
+    }
+}
+
 /// RTCPReader is used by Interceptor.bind_rtcpreader.
 #[async_trait]
 pub trait RTCPReader {
@@ -103,8 +165,24 @@ pub trait RTCPReader {
     async fn read(&self, buf: &mut [u8], attributes: &Attributes) -> Result<(usize, Attributes)>;
 }
 
-/// Attributes are a generic key/value store used by interceptors
-pub type Attributes = HashMap<usize, usize>;
+pub type RTCPReaderBoxFn = Box<
+    dyn (Fn(
+            &mut [u8],
+            &Attributes,
+        ) -> Pin<Box<dyn Future<Output = Result<(usize, Attributes)>> + Send + Sync>>)
+        + Send
+        + Sync,
+>;
+
+pub struct RTCPReaderFn(pub RTCPReaderBoxFn);
+
+#[async_trait]
+impl RTCPReader for RTCPReaderFn {
+    /// read a batch of rtcp packets
+    async fn read(&self, buf: &mut [u8], attributes: &Attributes) -> Result<(usize, Attributes)> {
+        self.0(buf, attributes).await
+    }
+}
 
 /// Helper for the tests.
 #[cfg(test)]
